package finance.chai.chai_webview_plugin;

import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.Build;
import android.webkit.ValueCallback;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.PluginRegistry;

/**
 * ChaiWebviewPlugin
 * <p>
 * modified Uther on 09/01/2020
 */
public class ChaiWebviewPlugin implements MethodCallHandler, WebViewClientListener {

    private static final String CHANNEL_NAME = "chai_webview_plugin";

    private static ChaiWebviewPlugin _instance;

    private WeakReference<Context> activity;
    private MethodChannel channel;

    public static ChaiWebviewPlugin getInstance() {
        return _instance;
    }

    public static void registerWith(PluginRegistry.Registrar registrar) {
        MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL_NAME);
        //registrar.activity() is flutter activity. maybe flutter app's MainActivity
        ChaiWebviewPlugin instance = new ChaiWebviewPlugin(registrar.activity(), channel);
        _instance = instance;
        channel.setMethodCallHandler(instance);
    }

    private ChaiWebviewPlugin(Context activity, MethodChannel channel) {
        this.activity = new WeakReference<>(activity);
        this.channel = channel;
    }

    @Override
    public void onMethodCall(MethodCall call, @NonNull MethodChannel.Result result) {
        switch (call.method) {
            case "launch":
                openUrl(call, result);
                break;
            case "close":
                close();
                break;
            case "eval":
                eval(call, result);
                break;
            case "resetAuthCookies":
                resetAuthCookies(call, result);
                break;
            case "reload":
                reload(result);
                break;
            case "back":
                back(result);
                break;
            case "forward":
                forward(result);
                break;
            case "hide":
                hide(result);
                break;
            case "show":
                show(result);
                break;
            case "reloadUrl":
                reloadUrl(call, result);
                break;
            case "stopLoading":
                stopLoading(call, result);
                break;
            case "cleanCookies":
                cleanCookies(result);
                break;
            case "getPlatformVersion":
                getPlatformVersion(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void getPlatformVersion(MethodCall call, MethodChannel.Result result) {
        result.success("Android " + Build.VERSION.RELEASE);
    }

    private void openUrl(MethodCall call, MethodChannel.Result result) {
        WebviewBuildParams webviewBuildParams = WebviewBuildParams.newIsntance(call);
        @Nullable Context context = activity.get();
        if (context != null) {
            Intent intent = WebViewActivity.newIntent(context, webviewBuildParams.url, webviewBuildParams);
            context.startActivity(intent);
            result.success(null);
        } else {
            //flutter app maybe destroyed
            //TODO 에러 정상으로 보내기
            result.error("e1", "e2", null);
        }
    }

    private void resetAuthCookies(MethodCall call, MethodChannel.Result result) {
        Map<String, String> authCookies = call.argument("authCookies");
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.setAuthCookies(authCookies);
        }
        result.success(null);
    }

    private void stopLoading(MethodCall call, MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.stop();
        }
        result.success(null);
    }

    private void close() {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.close();
        }
    }

    /**
     * Navigates back on the Webview.
     */
    private void back(MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.back();
        }
        result.success(null);
    }

    /**
     * Navigates forward on the Webview.
     */
    private void forward(MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.forward();
        }
        result.success(null);
    }

    /**
     * Reloads the Webview.
     */
    private void reload(MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.reload();
        }
        result.success(null);
    }

    private void reloadUrl(MethodCall call, MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            String url = call.argument("url");
            webViewInterface.reloadUrl(url);
        }
        result.success(null);
    }

    private void eval(MethodCall call, final MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            String code = call.argument("code");
            webViewInterface.eval(code, new ValueCallback<String>() {
                @Override
                public void onReceiveValue(String value) {
                    result.success(value);
                }
            });
        }
    }

    private void hide(final MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.hide();
        }
        result.success(null);
    }

    private void show(final MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.show();
        }
        result.success(null);
    }

    private void cleanCookies(final MethodChannel.Result result) {
        WebViewInterface webViewInterface = this.webViewInterface;
        if (webViewInterface != null) {
            webViewInterface.clearCookies();
        }
        result.success(null);
    }

    /**
     * WebView activity
     */
    public void onWebviewActivityDestroy() {
        Map<String, Object> visible = new HashMap<>();
        visible.put("visible", true);
        channel.invokeMethod("onDestroy", visible);
    }

    @Nullable
    private WebViewInterface webViewInterface = null;

    public void registerWebviewInterface(WebViewInterface webViewInterface) {
        this.webViewInterface = webViewInterface;
    }

    public void unregisterWebviewInterface() {
        this.webViewInterface = null;
    }

    ///// Webview Client Listener ////////

    @Override
    public void onPageStarted(String url, Bitmap favicon) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", url);
        data.put("type", "startLoad");
        channel.invokeMethod("onState", data);
    }

    @Override
    public void onPageFinished(String url) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", url);

        channel.invokeMethod("onUrlChanged", data);

        data.put("type", "finishLoad");
        channel.invokeMethod("onState", data);
    }

    @Override
    public void onShouldOverrideUrlLoading(String url, boolean invalid) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", url);
        data.put("type", invalid ? "abortLoad" : "shouldStart");
        channel.invokeMethod("onState", data);
    }

    @Override
    public void onReceivedError(int errorCode, String description, String failingUrl) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", failingUrl);
        data.put("code", errorCode);
        channel.invokeMethod("onHttpError", data);
    }
}
