package finance.chai.chai_webview_plugin;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.view.View;
import android.webkit.CookieManager;
import android.webkit.CookieSyncManager;
import android.webkit.GeolocationPermissions;
import android.webkit.JsResult;
import android.webkit.ValueCallback;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.Iterator;
import java.util.Map;

public class WebViewActivity extends Activity implements WebViewInterface {
    private final int REQ_FILE_CHOOSER = 0x01;

    private ChaiWebviewPlugin chaiWebviewPlugin;
    private WebView webView;

    private ValueCallback<Uri> uploadMessageCallback;
    private ValueCallback<Uri[]> uploadMessageArrayCallback;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

//        int hash = getIntent().getIntExtra("hash", 0);
//        chaiWebviewPlugin = ChaiWebviewPlugin.instances.get(hash);
//        webView = chaiWebviewPlugin.webViewManager.webView;
        chaiWebviewPlugin = ChaiWebviewPlugin.getInstance();
        webView = new WebView(this);
        webView.setVisibility(View.INVISIBLE);
        setContentView(webView);
        webView.requestFocus();

        WebViewClient webClient = new BrowserClient(chaiWebviewPlugin) {
            @Override
            public void onPageFinished(WebView view, String url) {
                super.onPageFinished(view, url);
                webView.setVisibility(View.VISIBLE);
            }
        };

        webView.setWebViewClient(webClient);
        webView.setWebChromeClient(new WebChromeClient() {
            private final String TAG = "WebviewClient";

            //The undocumented magic method override
            //Eclipse will swear at you if you try to put @Override here
            // For Android 3.0+
            public void openFileChooser(ValueCallback<Uri> uploadMsg) {

                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(null);
                }
                uploadMessageCallback = uploadMsg;
                Intent i = new Intent(Intent.ACTION_GET_CONTENT);
                i.addCategory(Intent.CATEGORY_OPENABLE);
                i.setType("image/*");
                startActivityForResult(Intent.createChooser(i, "File Chooser"), REQ_FILE_CHOOSER);

            }

            // For Android 3.0+
            public void openFileChooser(ValueCallback uploadMsg, String acceptType) {
                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(null);
                }
                uploadMessageCallback = uploadMsg;
                Intent i = new Intent(Intent.ACTION_GET_CONTENT);
                i.addCategory(Intent.CATEGORY_OPENABLE);
                i.setType("*/*");
                startActivityForResult(
                        Intent.createChooser(i, "File Browser"),
                        REQ_FILE_CHOOSER);
            }

            //For Android 4.1
            public void openFileChooser(ValueCallback<Uri> uploadMsg, String acceptType, String capture) {
                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(null);
                }
                uploadMessageCallback = uploadMsg;
                Intent i = new Intent(Intent.ACTION_GET_CONTENT);
                i.addCategory(Intent.CATEGORY_OPENABLE);
                i.setType("image/*");
                startActivityForResult(Intent.createChooser(i, "File Chooser"), REQ_FILE_CHOOSER);

            }

            //For Android 5.0+
            public boolean onShowFileChooser(
                    WebView webView, ValueCallback<Uri[]> filePathCallback,
                    FileChooserParams fileChooserParams) {

                if (uploadMessageArrayCallback != null) {
                    uploadMessageArrayCallback.onReceiveValue(null);
                }
                uploadMessageArrayCallback = filePathCallback;

                Intent contentSelectionIntent = new Intent(Intent.ACTION_GET_CONTENT);
                contentSelectionIntent.addCategory(Intent.CATEGORY_OPENABLE);
                contentSelectionIntent.setType("*/*");
                Intent[] intentArray;
                intentArray = new Intent[0];

                Intent chooserIntent = new Intent(Intent.ACTION_CHOOSER);
                chooserIntent.putExtra(Intent.EXTRA_INTENT, contentSelectionIntent);
                chooserIntent.putExtra(Intent.EXTRA_TITLE, "Image Chooser");
                chooserIntent.putExtra(Intent.EXTRA_INITIAL_INTENTS, intentArray);
                startActivityForResult(chooserIntent, REQ_FILE_CHOOSER);

                return true;
            }

            public boolean onJsConfirm(WebView view, String url, String message, final JsResult result) {

                new AlertDialog.Builder(WebViewActivity.this)
                        .setMessage(message)
                        .setPositiveButton(android.R.string.ok,
                                new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog, int which) {
                                        result.confirm();
                                    }
                                })
                        .setNegativeButton(android.R.string.cancel,
                                new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        result.cancel();
                                    }
                                })
                        .setCancelable(false)
                        .create()
                        .show();
                return true;
            }
        });

        chaiWebviewPlugin.registerWebviewInterface(this);

        Bundle extras = getIntent().getExtras();

        if (extras != null) {
            Map<String, String> headers = null;
            if (extras.containsKey(WEBVIEW_BUILD_PARAMS)) {
                WebviewBuildParams params = (WebviewBuildParams) extras.getSerializable(WEBVIEW_BUILD_PARAMS);
                if (params != null) {
                    initWebview(params);
                    headers = params.headers;
                }
            }
            if (extras.containsKey(URL_PARAM)) {
                String url = extras.getString(URL_PARAM);
                if (headers != null) {
                    webView.loadUrl(url, headers);
                } else {
                    webView.loadUrl(url);
                }
            }
        } else {
            //TODO maybe error
        }
    }

    private void initWebview(@NonNull WebviewBuildParams params) {
        WebSettings settings = webView.getSettings();
        settings.setJavaScriptEnabled(params.withJavascript);
        settings.setBuiltInZoomControls(params.withZoom);
        settings.setSupportZoom(params.withZoom);
        settings.setDomStorageEnabled(params.withLocalStorage);
        settings.setJavaScriptCanOpenWindowsAutomatically(params.supportMultipleWindows);
        settings.setSupportMultipleWindows(params.supportMultipleWindows);
        settings.setAppCacheEnabled(params.appCacheEnabled);
        settings.setAllowFileAccessFromFileURLs(params.allowFileURLs);
        settings.setAllowUniversalAccessFromFileURLs(params.allowFileURLs);
        settings.setUseWideViewPort(params.useWideViewPort);

        //TODO
//        webViewClient.updateInvalidUrlRegex(invalidUrlRegex);

        if (params.geolocationEnabled) {
            settings.setGeolocationEnabled(true);
            webView.setWebChromeClient(new WebChromeClient() {
                @Override
                public void onGeolocationPermissionsShowPrompt(String origin, GeolocationPermissions.Callback callback) {
                    callback.invoke(origin, true, false);
                }
            });
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            settings.setMixedContentMode(WebSettings.MIXED_CONTENT_COMPATIBILITY_MODE);
        }

        if (params.clearCache) {
            clearCache();
        }

        if (params.hidden) {
            webView.setVisibility(View.GONE);
        }

        if (params.clearCookies) {
            clearCookies();
        }

        if (params.userAgent != null) {
            settings.setUserAgentString(params.userAgent);
        }

        if (!params.scrollBar) {
            webView.setVerticalScrollBarEnabled(false);
        }

        if (params.authCookies != null) {
            setAuthCookies(params.authCookies);
        }
    }

    @Override
    public void onBackPressed() {
        if (webView.canGoBack()) {
            webView.goBack();
        } else {
            super.onBackPressed();
        }
    }


    @Override
    protected void onDestroy() {
        super.onDestroy();
        chaiWebviewPlugin.onWebviewActivityDestroy();
        chaiWebviewPlugin.unregisterWebviewInterface();
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQ_FILE_CHOOSER) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                Uri[] results = null;
                if (resultCode == Activity.RESULT_OK && data != null) {
                    String dataString = data.getDataString();
                    if (dataString != null) {
                        results = new Uri[]{Uri.parse(dataString)};
                    }
                }
                if (uploadMessageArrayCallback != null) {
                    uploadMessageArrayCallback.onReceiveValue(results);
                    uploadMessageArrayCallback = null;
                }
            } else {
                Uri result = null;
                if (resultCode == RESULT_OK && data != null) {
                    result = data.getData();
                }
                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(result);
                    uploadMessageCallback = null;
                }
            }
            return;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    ///////Intent//////////

    private final static String URL_PARAM = "URL_PARAM";
    private final static String WEBVIEW_BUILD_PARAMS = "WEBVIEW_BUILD_PARAMS";

    public static Intent newIntent(@NonNull Context context, @Nullable String url, @Nullable WebviewBuildParams params) {
        Intent intent = new Intent(context, WebViewActivity.class);
        Bundle bundle = new Bundle();
        bundle.putString(URL_PARAM, url);
        bundle.putSerializable(WEBVIEW_BUILD_PARAMS, params);
        intent.putExtras(bundle);
        return intent;
    }


    ///////WebViewInterface//////////


    @Override
    public void clearCache() {
        webView.clearCache(true);
        webView.clearFormData();
    }

    @Override
    public void clearCookies() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            CookieManager.getInstance().removeAllCookies(new ValueCallback<Boolean>() {
                @Override
                public void onReceiveValue(Boolean aBoolean) {

                }
            });
        } else {
            CookieManager.getInstance().removeAllCookie();
        }
    }

    @Override
    public void setAuthCookies(Map<String, String> authCookies) {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            CookieSyncManager.createInstance(webView.getContext());
        }
        CookieManager cookieManager = CookieManager.getInstance();
        cookieManager.setAcceptCookie(true);

        Iterator<String> it = authCookies.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            String value = authCookies.get(key);
            cookieManager.setCookie(".finance.chai", key + "=" + value);
        }
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            CookieSyncManager.getInstance().sync();
        }
    }

    @Override
    public void reloadUrl(String url) {
        webView.loadUrl(url);
    }

    @Override
    public void close() {
        finish();
    }

    @Override
    public void eval(String code, ValueCallback<String> callback) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
            webView.evaluateJavascript(code, callback);
        }
    }

    @Override
    public void reload() {
        webView.reload();
    }

    @Override
    public void back() {
        webView.goBack();
    }

    @Override
    public void forward() {
        if (webView.canGoForward()) {
            webView.goForward();
        }
    }

    @Override
    public void resize(FrameLayout.LayoutParams params) {
        webView.setLayoutParams(params);
    }

    @Override
    public boolean canGoBack() {
        return webView.canGoBack();
    }

    @Override
    public boolean canGoForward() {
        return webView.canGoForward();
    }

    @Override
    public void hide() {
        webView.setVisibility(View.GONE);
    }

    @Override
    public void stop() {
        webView.setVisibility(View.VISIBLE);
    }

    @Override
    public void show() {
        webView.stopLoading();
    }
}
